/**
This file is part of a jTEM project.
All jTEM projects are licensed under the FreeBSD license 
or 2-clause BSD license (see http://www.opensource.org/licenses/bsd-license.php). 

Copyright (c) 2005-2010, Technische Universität Berlin, jTEM
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

-	Redistributions of source code must retain the above copyright notice, 
	this list of conditions and the following disclaimer.

-	Redistributions in binary form must reproduce the above copyright notice, 
	this list of conditions and the following disclaimer in the documentation 
	and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS 
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
OF SUCH DAMAGE.
**/

package de.jtem.function;

/** This class provides interfaces to be implemented by classes modelling
 * functions defined on various mathematically important standard spaces. The
 * target space of these functions will be specified by
 * subinterfaces. The domains selected here have been chosen according to the
 * following criteria: The domains should include
 * <ul>
 * <li>
 * The set of integers between 0 and n-1. This is the canonical choice of a
 * finite index set and maps from such a domain to some space A are usually
 * called "n-tuples of elements in A".
 * <li>
 * The set of non-negative integers. Maps from the non-negative integers to
 * some space A are usually called "sequences of Elements in A".
 * <li>
 * The set of integers.
 * <li>
 * The closed interval [a,b] between two real numbers a and b. Maps from the
 * such an interval into some space A can be called "paths in A".
 * <li>
 * The set of non-negative real numbers.
 * <li>
 * The real line.
 * <li>
 * The circle S (the only connected one-dimensional compact manifold).
 * Functions on S are described by periodic functions on the real line.
 * <li>
 * The "discrete circle" (a cylic graph). Functions on the discrete
 * circle are described by periodic sequences.
 * </ul>
 * Furthermore we provide some Cartesian products of two or three of the above
 * spaces. In the case of the cartesian product of two finite sets of indices
 * the maps from such a domain to a set A can be called "matrices of elements
 * in A". In analogy, we adopt matrix terminology for the naming of methods by
 * refering to the first argument of a function on a Cartesian product as
 * the "row" argument and the second as the "column" argument. In the case of a
 * threefold cartesian product the third argument is said to specify the
 * "layer" which means we imagine a stack of matrices (two dimensional
 * arrangements of elements of A) with several layers in a third dimension.
 *<p>
 * The actual choice of product spaces was guided by the following principles:
 * <ul>
 * <li>
 * To allow for interpolation, replacing a factor corresponding to an integer
 * argument by one corresponding to a real argument should always be possible.
 * <li>
 * If one of the factors has an interpretation of "time" (like in a dynamical
 * system or evolution equation), then we demand it to be the first factor.
 * Since the main motivation for us to include the non-negative integers and the
 * non-negative reals is in fact to serve as the "time" of some irreversible
 * dynamical system (like iterating a non-invertible function or solving a
 * parabolic PDE), these two spaces only occur as the first factor.
 * <li>
 * Periodicity is natural only for spacelike variables, so periodic factors
 * only occur at the end. "Periodic" will then mean "periodic in the last
 * variable", "doubly periodic" will mean "periodic in the last two variables".
 * <li>
 * To keep the number of interfaces within reasonable bounds, mixing of integer
 * and real factors is not supported in the case of three factors. Moreover, the
 * last two factors must be of the same type.
 * </ul>
 */

public abstract class Domain {

    private interface WithFiniteNumberOfRows {

        /** Returns the number of rows (the range of the first variable of
         * this function). */

        public int getNumRows();
    }

    private interface WithFiniteNumberOfColumns {

        /** Returns the number of columns (the range of the second variable of
         * this function). */

        public int getNumColumns();
    }

    private interface WithFiniteNumberOfLayers {

        /** Returns the number of layers (the range of the third variable of
         * this function). */

        public int getNumLayers();
    }

    private interface WithIntervalOfRows {

        /** Returns the lower bound of the closed interval that is the range
         * of the first argument of this function. The result
         * is guaranteed to be smaller or equal to the upper bound.
         */

        public double getLowerBoundOfRowArgument();

        /** Returns the upper bound of the closed interval that is the range
         * of the first argument of this function. The result
         * is guaranteed to be greater or equal to the lower bound.
         */

        public double getUpperBoundOfRowArgument();
    }

    private interface WithIntervalOfColumns {

        /** Returns the lower bound of the closed interval that is the range
         * of the second argument of this function. The result
         * is guaranteed to be smaller or equal to the upper bound.
         */

        public double getLowerBoundOfColumnArgument();

        /** Returns the upper bound of the closed interval that is the range
         * of the second argument of this function. The result
         * is guaranteed to be greater or equal to the lower bound.
         */

        public double getUpperBoundOfColumnArgument();
    }

    private interface WithIntervalOfLayers {

        /** Returns the lower bound of the closed interval that is the range
         * of the third argument of this function. The result
         * is guaranteed to be smaller or equal to the upper bound.
         */

        public double getLowerBoundOfLayerArgument();

        /** Returns the upper bound of the closed interval that is the range
         * of the third argument of this function. The result
         * is guaranteed to be greater or equal to the lower bound.
         */

        public double getUpperBoundOfLayerArgument();
    }

    private interface IntegerPeriodic {

        /** Returns the period of this function (a positive integer),
         * refering to the behaviour in the last coordinate. */

        public int getPeriod();
    }

    private interface IntegerDoublyPeriodic
        extends IntegerPeriodic {

        /** Returns the second period of this function (a positive integer),
         * refering to the behaviour in the last but one coordinate. */

        public int getSecondPeriod();
    }

    private interface RealPeriodic {

        /** Returns the period of this function (a positive real number),
         * refering to the behaviour in the last coordinate. */

        public double getPeriod();
    }

    private interface RealDoublyPeriodic
        extends RealPeriodic {

            /** Returns the second period of this function (a positive real number),
         * refering to the behaviour in the last but one coordinate. */

        public double getSecondPeriod();
    }

    /** This interface models mathematical objects (like maps into some other
     * set or difference equations) that have as their domain of definition
     * the set of integers between zero and n-1 (elements of which are
     * referered to as "indices). */

    public interface OnIndex {

        /** Returns the number of indices (a non-negative integer). */

        public int getNumIndices();
    }

    /** This interface models mathematical objects (like maps into some other
     * set or difference equations) that have as their domain of definition
     * the set of non-negative integers. */

    public interface OnNonNegativeIntegers {}

    /** This interface models mathematical objects (like maps into some other
     * set or difference equations) that have the integers as their domain of
     * definition. */

    public interface OnIntegers {}

    /** This interface models mathematical objects (like maps into some other
     * set or difference equations) that have the integers as their domain of
     * definition and which are periodic. */

    public interface PeriodicOnIntegers
        extends IntegerPeriodic, OnIntegers {}

    /** This interface models mathematical objects (like maps into some other
     * set or differential equations) that have a closed interval of real
     * numbers as their domain of definition. */

    public interface OnInterval {

        /** Returns the lower bound of the interval of definition. The result
         * is guaranteed to be smaller or equal to the upper bound. */

        public double getLowerBoundOfInterval();

        /** Returns the upper bound of the interval of definition. The result
         * is guaranteed to be greater or equal to the lower bound. */

        public double getUpperBoundOfInterval();
    }

    /** This interface models mathematical objects (like maps into some other
     * set or difference equations) that have the non-negative real numbers
     * as their domain of definition. */

    public interface OnNonNegativeReals {}

    /** This interface models mathematical objects (like maps into some other
         * set or differential equations) that have the real line as their domain of
     * definition. */

    public interface OnReals {}

    /** This interface models mathematical objects (like maps into some other
         * set or differential equations) that have the real line as their domain of
     * definition and which are periodic. */

    public interface PeriodicOnReals
        extends RealPeriodic, OnReals {}

    /** This interface models mathematical objects (like maps into some other
     * set or difference equations) that have as their domain of definition
     * the cartesian product of two finite sets of indices. */

    public interface OnIndexCrossIndex
        extends WithFiniteNumberOfRows, WithFiniteNumberOfColumns {}

    /** This interface models mathematical objects (like maps into some other
     * set or difference equations) that have as their domain of definition
     * the cartesian product of a finite set of indices with the set of
     * integers. */

    public interface OnIndexCrossIntegers
        extends WithFiniteNumberOfRows {}

    /** This interface models mathematical objects (like maps into some other
     * set or difference equations) that have as their domain of definition
     * the cartesian product of a finite set of indices with the set of
     * integers and which are periodic in the second variable. */

    public interface PeriodicOnIndexCrossIntegers
        extends IntegerPeriodic, OnIndexCrossIntegers {}

    /** This interface models mathematical objects (like maps into some other
     * set or difference equations) that have as their domain of definition
     * the cartesian product of a finite set of indices
     * with a closed interval on the real line. */

    public interface OnIndexCrossInterval
        extends WithFiniteNumberOfRows, WithIntervalOfColumns {}

    /** This interface models mathematical objects (like maps into some other
     * set or difference equations) that have as their domain of definition
     * the cartesian product of a finite set of indices with the real line. */

    public interface OnIndexCrossReals
        extends WithFiniteNumberOfRows {}

    /** This interface models mathematical objects (like maps into some other
     * set or difference equations) that have as their domain of definition
     * the cartesian product of a finite set of indices with the real line
     * and which are periodic in the second variable. */

    public interface PeriodicOnIndexCrossReals
        extends RealPeriodic, OnIndexCrossReals {}

    /** This interface models mathematical objects (like maps into some other
     * set or difference equations) that have as their domain of definition
     * the cartesian product of the set of
     * non-negative integers with a finite set of indices. */

    public interface OnNonNegativeIntegersCrossIndex
        extends WithFiniteNumberOfColumns {}

    /** This interface models mathematical objects (like maps into some other
     * set or difference equations) that have as their domain of definition
     * the cartesian product of the set of
     * non-negative integers with the set of integers. */

    public interface OnNonNegativeIntegersCrossIntegers {}

    /** This interface models mathematical objects (like maps into some other
     * set or difference equations) that have as their domain of definition
     * the cartesian product of the set of non-negative integers with the
     * integers and which are periodic in the second variable. */

    public interface PeriodicOnNonNegativeIntegersCrossIntegers
        extends OnNonNegativeIntegersCrossIntegers,
        IntegerPeriodic {}

    /** This interface models mathematical objects (like maps into some other
     * set or difference equations) that have as their domain of definition
     * the cartesian product of the set of
     * non-negative integers with a closed interval on the real line. */

    public interface OnNonNegativeIntegersCrossInterval
        extends WithIntervalOfColumns {}

    /** This interface models mathematical objects (like maps into some other
     * set or difference equations) that have as their domain of definition
     * the cartesian product of the set of
     * non-negative integers with the real line. */

    public interface OnNonNegativeIntegersCrossReals {}

    /** This interface models mathematical objects (like maps into some other
     * set or difference equations) that have as their domain of definition
     * the cartesian product of the set of non-negative integers with the real
     * line and which are periodic in the second variable. */

    public interface PeriodicOnNonNegativeIntegersCrossReals
        extends OnNonNegativeIntegersCrossReals, RealPeriodic {}

    /** This interface models mathematical objects (like maps into some other
     * set or difference equations) that have as their domain of definition
     * the cartesian product of the set of
     * integers with a finite set of indices. */

    public interface OnIntegersCrossIndex
        extends OnNonNegativeIntegersCrossIndex {}

    /** This interface models mathematical objects (like maps into some other
     * set or difference equations) that have as their domain of definition
     * the cartesian product of the set of
     * integers with itself. */

    public interface OnIntegersCrossIntegers
        extends OnNonNegativeIntegersCrossIntegers {}

    /** This interface models mathematical objects (like maps into some other
     * set or difference equations) that have as their domain of definition
     * the cartesian product of the set of integers with itself and which are
     * periodic in the second variable. */

    public interface PeriodicOnIntegersCrossIntegers
        extends PeriodicOnNonNegativeIntegersCrossIntegers,
        OnIntegersCrossIntegers, IntegerPeriodic {}

    /** This interface models mathematical objects (like maps into some other
     * set or difference equations) that have as their domain of definition
     * the cartesian product of the set of integers with itself and which are
     * periodic in the both variables. */

    public interface DoublyPeriodicOnIntegersCrossIntegers
        extends IntegerDoublyPeriodic, PeriodicOnIntegersCrossIntegers {}

    /** This interface models mathematical objects (like maps into some other
     * set or difference equations) that have as their domain of definition
     * the cartesian product of the set of
     * integers with a closed interval on the real line. */

    public interface OnIntegersCrossInterval
        extends OnNonNegativeIntegersCrossInterval, WithIntervalOfColumns {}

    /** This interface models mathematical objects (like maps into some other
     * set or difference equations) that have as their domain of definition
     * the cartesian product of the set of
     * integers with the real line. */

    public interface OnIntegersCrossReals
        extends OnNonNegativeIntegersCrossReals {}

    /** This interface models mathematical objects (like maps into some other
     * set or difference equations) that have as their domain of definition
     * the cartesian product of the set of non-negative integers with the real
     * line and which are periodic in the second variable. */

    public interface PeriodicOnIntegersCrossReals
        extends PeriodicOnNonNegativeIntegersCrossReals {}

    /** This interface models mathematical objects (like maps into some other
     * set or difference equations) that have as their domain of definition
     * the cartesian product of a closed interval with a finite sets of
     * indices. */

    public interface OnIntervalCrossIndex
        extends WithIntervalOfRows, WithFiniteNumberOfColumns {}

    /** This interface models mathematical objects (like maps into some other
     * set or difference equations) that have as their domain of definition
     * the cartesian product of a closed interval with the set of
     * integers. */

    public interface OnIntervalCrossIntegers
        extends WithIntervalOfRows {}

    /** This interface models mathematical objects (like maps into some other
     * set or difference equations) that have as their domain of definition
     * the cartesian product of a closed interval with the set of
     * integers and which are periodic in the second variable. */

    public interface PeriodicOnIntervalCrossIntegers
        extends IntegerPeriodic, OnIntervalCrossIntegers {}

    /** This interface models mathematical objects (like maps into some other
     * set or difference equations) that have as their domain of definition
     * the cartesian product of a closed interval on the real line
     * with another closed interval. */

    public interface OnIntervalCrossInterval
        extends WithIntervalOfRows, WithIntervalOfColumns {}

    /** This interface models mathematical objects (like maps into some other
     * set or difference equations) that have as their domain of definition
     * the cartesian product of a closed interval with the real line. */

    public interface OnIntervalCrossReals
        extends WithIntervalOfRows {}

    /** This interface models mathematical objects (like maps into some other
     * set or difference equations) that have as their domain of definition
     * the cartesian product of a closed interval with the real line
     * and which are periodic in the second variable. */

    public interface PeriodicOnIntervalCrossReals
        extends RealPeriodic, OnIntervalCrossReals {}

    /** This interface models mathematical objects (like maps into some other
     * set or difference equations) that have as their domain of definition
     * the cartesian product of the set of
     * non-negative real numbers with a finite set of indices. */

    public interface OnNonNegativeRealsCrossIndex
        extends WithFiniteNumberOfColumns {}

    /** This interface models mathematical objects (like maps into some other
     * set or difference equations) that have as their domain of definition
     * the cartesian product of the set of
     * non-negative real numbers with the set of integers. */

    public interface OnNonNegativeRealsCrossIntegers {}

    /** This interface models mathematical objects (like maps into some other
     * set or difference equations) that have as their domain of definition
     * the cartesian product of the set of non-negative real numbers with the
     * integers which are periodic in the second variable. */

    public interface PeriodicOnNonNegativeRealsCrossIntegers
        extends OnNonNegativeRealsCrossIntegers,
        IntegerPeriodic {}

    /** This interface models mathematical objects (like maps into some other
     * set or difference equations) that have as their domain of definition
     * the cartesian product of the set of
     * non-negative real numbers with a closed interval on the real line. */

    public interface OnNonNegativeRealsCrossInterval
        extends WithIntervalOfColumns {}

    /** This interface models mathematical objects (like maps into some other
     * set or difference equations) that have as their domain of definition
     * the cartesian product of the set of
     * non-negative real numbers with the real line. */

    public interface OnNonNegativeRealsCrossReals {}

    /** This interface models mathematical objects (like maps into some other
     * set or difference equations) that have as their domain of definition
     * the cartesian product of the set of non-negative real numbers with the
     * real line and which are periodic in the second variable. */

    public interface PeriodicOnNonNegativeRealsCrossReals
        extends OnNonNegativeRealsCrossReals, RealPeriodic {}

    /** This interface models mathematical objects (like maps into some other
     * set or difference equations) that have as their domain of definition
     * the cartesian product of the real line with a finite set of indices. */

    public interface OnRealsCrossIndex
        extends OnNonNegativeRealsCrossIndex {}

    /** This interface models mathematical objects (like maps into some other
     * set or difference equations) that have as their domain of definition
     * the cartesian product of the real line with the set of integers. */

    public interface OnRealsCrossIntegers
        extends OnNonNegativeRealsCrossIntegers {}

    /** This interface models mathematical objects (like maps into some other
     * set or difference equations) that have as their domain of definition
         * the cartesian product of the real line with the set of integers and which
     * are periodic in the second variable. */

    public interface PeriodicOnRealsCrossIntegers
        extends PeriodicOnNonNegativeRealsCrossIntegers,
        OnRealsCrossIntegers, IntegerPeriodic {}

    /** This interface models mathematical objects (like maps into some other
     * set or difference equations) that have as their domain of definition
     * the cartesian product of the real line with a closed interval. */

    public interface OnRealsCrossInterval
        extends OnNonNegativeRealsCrossInterval, WithIntervalOfColumns {}

    /** This interface models mathematical objects (like maps into some other
     * set or difference equations) that have as their domain of definition
     * the real plane. */

    public interface OnRealsCrossReals
        extends OnNonNegativeRealsCrossReals {}

    /** This interface models mathematical objects (like maps into some other
     * set or difference equations) that have as their domain of definition
     * the real plane and which are periodic in the second variable. */

    public interface PeriodicOnRealsCrossReals
        extends PeriodicOnNonNegativeIntegersCrossReals {}

    /** This interface models mathematical objects (like maps into some other
     * set or difference equations) that have as their domain of definition
     * the real plane and which are periodic in both variables. */

    public interface DoublyPeriodicOnRealsCrossReals
        extends PeriodicOnRealsCrossReals {}

    /** This interface models mathematical objects (like maps into some other
     * set or difference equations) that have as their domain of definition
     * the Cartesian product of three fine sets of indices. */

    public interface OnIndexCrossIndexCrossIndex
        extends WithFiniteNumberOfRows,
        WithFiniteNumberOfColumns, WithFiniteNumberOfLayers {}

    /** This interface models mathematical objects (like maps into some other
     * set or difference equations) that have as their domain of definition
     * the Cartesian product of a fine set of indices with the integer plane.
     */

    public interface OnIndexCrossIntegersCrossIntegers
        extends WithFiniteNumberOfRows {}

    /** This interface models mathematical objects (like maps into some other
     * set or difference equations) that have as their domain of definition
     * the Cartesian product of a fine set of indices with the integer plane
     * and which are periodic in the last two arguments. */

    public interface DoublyPeriodicOnIndexCrossIntegersCrossIntegers
        extends WithFiniteNumberOfLayers, IntegerDoublyPeriodic {}

    /** This interface models mathematical objects (like maps into some other
     * set or difference equations) that have as their domain of definition
     * the Cartesian product of the non-negative integers with two finite sets
     * of indices. */

    public interface OnNonNegativeIntegersCrossIndexCrossIndex
        extends WithFiniteNumberOfColumns, WithFiniteNumberOfLayers {}

    /** This interface models mathematical objects (like maps into some other
     * set or difference equations) that have as their domain of definition
     * the Cartesian product of the non-negative integers with the
     * integer plane. */

    public interface OnNonNegativeIntegersCrossIntegersCrossIntegers {}

    /** This interface models mathematical objects (like maps into some other
     * set or difference equations) that have as their domain of definition
     * the Cartesian product of the non-negative integers with the integer
     * plane and which are periodic in the last two arguments. */

    public interface
        DoublyPeriodicOnNonNegativeIntegersCrossIntegersCrossIntegers
        extends IntegerDoublyPeriodic {}

    /** This interface models mathematical objects (like maps into some other
     * set or difference equations) that have as their domain of definition
     * the Cartesian product of the non-negative integers with two finite sets
     * of indices. */

    public interface OnIntegersCrossIndexCrossIndex
        extends OnNonNegativeIntegersCrossIndexCrossIndex {}

    /** This interface models mathematical objects (like maps into some other
     * set or difference equations) that have as their domain of definition
     * the Cartesian product of the non-negative integers with the
     * integer plane. */

    public interface OnIntegersCrossIntegersCrossIntegers
        extends OnNonNegativeIntegersCrossIntegersCrossIntegers {}

    /** This interface models mathematical objects (like maps into some other
     * set or difference equations) that have as their domain of definition
     * the Cartesian product of the non-negative integers with the integer
     * plane and which are periodic in the last two arguments. */

    public interface
        DoublyPeriodicOnIntegersCrossIntegersCrossIntegers
        extends DoublyPeriodicOnNonNegativeIntegersCrossIntegersCrossIntegers {}

    /** This interface models mathematical objects (like maps into some other
     * set or difference equations) that have as their domain of definition
     * the cartesian product of three copies of the integers and which are
     * periodic in all three variables. */

    public interface TriplyPeriodicOnIntegersCrossIntegersCrossIntegers
        extends DoublyPeriodicOnIntegersCrossIntegersCrossIntegers {

        /** Returns the third period of this function (a positive integer),
         * refering to the behaviour in the first coordinate. */

        public int getThirdPeriod();
    }


    /** This interface models mathematical objects (like maps into some other
     * set or difference equations) that have as their domain of definition
     * the Cartesian product of three closed intervals. */

    public interface OnIntervalCrossIntervalCrossInterval
        extends WithIntervalOfRows,
        WithIntervalOfColumns, WithIntervalOfLayers {}

    /** This interface models mathematical objects (like maps into some other
     * set or difference equations) that have as their domain of definition
     * the Cartesian product of a closed interval with the real plane.
     */

    public interface OnIntervalCrossRealsCrossReals
        extends WithIntervalOfRows {}

    /** This interface models mathematical objects (like maps into some other
     * set or difference equations) that have as their domain of definition
     * the Cartesian product of a closed interval with the real plane
     * and which are periodic in the last two arguments. */

    public interface DoublyPeriodicOnIntervalCrossRealsCrossReals
        extends WithIntervalOfRows, IntegerDoublyPeriodic {}

    /** This interface models mathematical objects (like maps into some other
     * set or difference equations) that have as their domain of definition
     * the Cartesian product of the non-negative real numbers with two closed
     * intervals. */

    public interface OnNonNegativeRealsCrossIntervalCrossInterval
        extends WithIntervalOfColumns, WithIntervalOfLayers {}

    /** This interface models mathematical objects (like maps into some other
     * set or difference equations) that have as their domain of definition
     * the Cartesian product of the non-negative real numbers with the
     * real plane. */

    public interface OnNonNegativeRealsCrossRealsCrossReals {}

    /** This interface models mathematical objects (like maps into some other
     * set or difference equations) that have as their domain of definition
     * the Cartesian product of the non-negative real numbers with the real
     * plane and which are periodic in the last two arguments. */

    public interface
        DoublyPeriodicOnNonNegativeRealsCrossRealsCrossReals
        extends RealDoublyPeriodic {}

    /** This interface models mathematical objects (like maps into some other
     * set or difference equations) that have as their domain of definition
     * the Cartesian product of the non-negative real numbers with two closed
     * intervals. */

    public interface OnRealsCrossIntervalCrossInterval
        extends OnNonNegativeRealsCrossIntervalCrossInterval {}

    /** This interface models mathematical objects (like maps into some other
     * set or difference equations) that have as their domain of definition
     * the Cartesian product of three copies of the real line. */

    public interface OnRealsCrossRealsCrossReals
        extends OnNonNegativeRealsCrossRealsCrossReals {}

    /** This interface models mathematical objects (like maps into some other
     * set or difference equations) that have as their domain of definition
     * the Cartesian product of three copies of the real line and which are
     * periodic in the last two arguments. */

    public interface
        DoublyPeriodicOnRealsCrossRealsCrossReals
        extends DoublyPeriodicOnNonNegativeRealsCrossRealsCrossReals {}

    /** This interface models mathematical objects (like maps into some other
     * set or difference equations) that have as their domain of definition
     * the Cartesian product of three copies of the real line and which are
     * periodic in the all three arguments. */

    public interface
        TriplyPeriodicOnRealsCrossRealsCrossReals
        extends DoublyPeriodicOnRealsCrossRealsCrossReals {

        /** Returns the third period of this function (a positive real  number),
         * refering to the behaviour in the first coordinate. */

        public double getThirdPeriod();
    }
}
