/**
This file is part of a jTEM project.
All jTEM projects are licensed under the FreeBSD license 
or 2-clause BSD license (see http://www.opensource.org/licenses/bsd-license.php). 

Copyright (c) 2005-2010, Technische Universität Berlin, jTEM
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

-	Redistributions of source code must retain the above copyright notice, 
	this list of conditions and the following disclaimer.

-	Redistributions in binary form must reproduce the above copyright notice, 
	this list of conditions and the following disclaimer in the documentation 
	and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS 
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
OF SUCH DAMAGE.
**/

package de.jtem.function.complex;

import de.jtem.mfc.field.Complex;


/** An implementation of the interface
 * <code>ComplexFunction.OnIndex</code>. Function values are stored
 * internally, so using this class to make a copy of a
 * <code>ComplexFunction.OnIndex c</code> (by the corresponding
 * constructor or the assign method) will make a snapshot of the current state
 * of <code>c</code>. No reference to <code>c</code> itself is retained. */

public class ComplexFunctionOnIndex implements ComplexFunction.OnIndex {

	Complex[] values = new Complex[0];

	public int getNumIndices() {
		return values.length;
	}

	public Complex valueAt(int j) {
		return values[j].copy();
	}

	/* Creates a tupel of length one and value zero. */

	public ComplexFunctionOnIndex() {
		this(1);
	}

	/** Creates a tupel of <code>n</code> zeroes. */

	public ComplexFunctionOnIndex(int n) {
		assignZero(n);
	}

	/** Creates a copy of the tupel <code>c</code> */

	public ComplexFunctionOnIndex(ComplexFunction.OnIndex c) {
		assign(c);
	}

	/** Interpret the array <code>c</code> as an n-tupel of complex numbers
	 * and create a copy of it. */

	public ComplexFunctionOnIndex(Complex[] c) {
		assign(c);
	}

	/** Keep the number of indices but set all values to zero. */

	public void assignZero() {
		for (int j = 0; j < values.length; j++) {
			values[j].assign(0, 0);
		}
	}

	/** Become a tupel of <code>n</code> zeroes. */

	public void assignZero(int n) {
		if (n != values.length) {
			setNumIndices(n);
		} else {
			assignZero();
		}
	}

	/** Become a copy of the tupel <code>c</code> */

	public void assign(ComplexFunction.OnIndex c) {
		setNumIndices(c.getNumIndices());
		for (int j = 0; j < values.length; j++) {
			values[j].assign(c.valueAt(j));
		}
	}

	/** Do nothing if <code>n</code> is equal to the current number of
	 * indices, become a tupel of <code>n</code> zeroes otherwise.
	 * @param n must at least one. */
	
	public void setNumIndices(int n) {
		if (n != values.length) {
			values = new Complex[n];
			for (int j = 0; j < n; j++) {
				values[j] = new Complex();
			}
		}
	}

	/** Interpret the array <code>c</code> as an n-tupel of Complex numbers
	 * and become a copy of it. */

	public void assign(Complex[] c) {
		setNumIndices(c.length);
		for (int j = 0; j < values.length; j++) {
			values[j].assign(c[j]);
		}
	}

	/** Set the value at index j to x. */

	public void setValueAt(int j, Complex x) {
		values[j].assign(x);
	}
}