/**
This file is part of a jTEM project.
All jTEM projects are licensed under the FreeBSD license 
or 2-clause BSD license (see http://www.opensource.org/licenses/bsd-license.php). 

Copyright (c) 2005-2010, Technische Universität Berlin, jTEM
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

-	Redistributions of source code must retain the above copyright notice, 
	this list of conditions and the following disclaimer.

-	Redistributions in binary form must reproduce the above copyright notice, 
	this list of conditions and the following disclaimer in the documentation 
	and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS 
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
OF SUCH DAMAGE.
**/

package de.jtem.function.complex;

import de.jtem.mfc.field.Complex;


/** An implementation of the interface
 * <code>ComplexFunction.OnIndexCrossIndex</code>. Function values are stored
 * internally, so using this class to make a copy of a
 * <code>ComplexFunction.OnIndexCrossIndex c</code> (by the corresponding
 * constructor or the assign method) will make a snapshot of the current state
 * of <code>c</code>. No reference to <code>c</code> itself is retained. */

public class ComplexFunctionOnIndexCrossIndex
		implements
			ComplexFunction.OnIndexCrossIndex {

	Complex[][] values= new Complex[0][0];

	public int getNumRows() {
		return values.length;
	}

	public int getNumColumns() {
		return values[0].length;
	}

	public Complex valueAt(int i, int j) {
		return values[i][j];
	}

	/** Creates a matrix with 1 row and 1 column. */

	public ComplexFunctionOnIndexCrossIndex() {
		assignZero(1,1);
	}

	/** Creates a matrix with <code>m</code> rows and <code>n</code> columns
	 * and all values zero.
	 * @param m must be at least one.
	 * @param n must be at least one. */

	public ComplexFunctionOnIndexCrossIndex(int m, int n) {
		assignZero(m, n);
	}

	/** Creates a copy of the function <code>c</code> */

	public ComplexFunctionOnIndexCrossIndex(ComplexFunction.OnIndexCrossIndex c) {
		assign(c);
	}

	/** Interprets the array <code>c</code> as a function of two indices
	 * and creates a copy of it.
	 * @param c must be a rectangular array and c[0][0] must be a non-null */

	public ComplexFunctionOnIndexCrossIndex(Complex[][] c) {
		assign(c);
	}

	/** Keep the number of rows and columns but set all values to zero. */

	public void assignZero() {
		for (int i = 0; i < values.length; i++) {
			for (int j = 0; j < values[0].length; j++) {
				values[i][j].assign(0,0);
			}
		}
	}

	/** Become a matrix with <code>m</code> rows and <code>n</code> columns
	 * and all values zero.
	 * @param m must be at least one.
	 * @param n must be at least one. */

	public void assignZero(int m, int n) {
		if (m != values.length || n != values[0].length) {
			setSize(m, n);
		} else {
			assignZero();
		}
	}

	/** Become a copy of the function <code>c</code> */

	public void assign(ComplexFunction.OnIndexCrossIndex c) {
		setSize(c.getNumRows(), c.getNumColumns());
		for (int i = 0; i < values.length; i++) {
			for (int j = 0; j < values[0].length; j++) {
				values[i][j].assign(c.valueAt(i, j));
			}
		}
	}

	/** Do nothing if <code>m</code> and <code>n</code>are equal to the
	 * current numbers of rows and columns, otherwise become a matrix with
	 * <code>m</code> rows and <code>n</code> columns and all values zero.
	 * @param m must be at least one.
	 * @param n must be at least one. */

	public void setSize(int m, int n) {
		if (m != values.length || n != values[0].length) {
			values = new Complex[m][n];
			for(int i=0; i<m; i++) {
				for (int j=0; j<n; j++) {
					values[i][j] = new Complex();
				}
			}
		}
	}

	/** Interpret the array <code>c</code> as a function of two indices
	 * and become a copy of it.
	 */

	public void assign(Complex[][] c) {
		int m = c.length;
		int n = c[0].length;
		setSize(m, n);
		for (int i = 0; i < m; i++) {
			for (int j = 0; j < n; j++) {
				values[i][j].assign(c[i][j]);
			}
		}
	}

	/** Sets the value at index <code>i,j</code> to <code>z</code>. */

	public void setValueAt(int i, int j, Complex z) {
		values[i][j].assign(z);
	}
}

