/**
This file is part of a jTEM project.
All jTEM projects are licensed under the FreeBSD license 
or 2-clause BSD license (see http://www.opensource.org/licenses/bsd-license.php). 

Copyright (c) 2005-2010, Technische Universität Berlin, jTEM
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

-	Redistributions of source code must retain the above copyright notice, 
	this list of conditions and the following disclaimer.

-	Redistributions in binary form must reproduce the above copyright notice, 
	this list of conditions and the following disclaimer in the documentation 
	and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS 
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
OF SUCH DAMAGE.
**/

package de.jtem.function.complex;

import de.jtem.mfc.field.Complex;


/** An implementation of the interface
 * <code>ComplexFunction.PeriodicOnIntegers</code>. Function values are stored
 * internally, so using this class to make a copy of a
 * <code>ComplexFunction.PeriodicOnIntegers c</code> (by the corresponding
 * constructor or the assign method) will make a snapshot of the current state
 * of <code>c</code>. No reference to <code>c</code> itself is retained. */

public class ComplexFunctionPeriodicOnIntegers
		implements
			ComplexFunction.PeriodicOnIntegers {

	ComplexFunctionOnIndex fundamentalDomain = new ComplexFunctionOnIndex();

	public ComplexFunctionPeriodicOnIntegers() {
	}

	/** Creates a copy of <code>c</code>. */

	public ComplexFunctionPeriodicOnIntegers(
			ComplexFunction.PeriodicOnIntegers c) {
		assign(c);
	}

	/** Creates a periodic sequence of period <code>n</code> with all values
	 * equal to zero. */

	public ComplexFunctionPeriodicOnIntegers(int n) {
		assignZero(n);
	}

	public int getPeriod() {
		return fundamentalDomain.getNumIndices();
	}

	public Complex valueAt(int j) {
		int n = fundamentalDomain.getNumIndices();
		int k = j%n;
		if (k<0) k+=n;
		return fundamentalDomain.valueAt(k);
	}
	
	/** Do nothing if <code>n</code> is equal to the current period, become
	 * a periodic sequence with period code>n</code> and all values zero
	 *  otherwise.
	 * @param n must be at least one. */
	
	public void setPeriod(int n) {
		fundamentalDomain.setNumIndices(n);
	}
	
	/** Become a copy of <code>c</code>. */

	public void assign(ComplexFunction.PeriodicOnIntegers c) {
		int n = c.getPeriod();
		if (n != getPeriod()) {
			fundamentalDomain.assignZero(n);
		}
		for (int j = 0; j < n; j++) {
			fundamentalDomain.setValueAt(j, c.valueAt(j));
		}
	}

	/** Become the periodic extension of <code>c</code>. */
	
	public void assign(ComplexFunction.OnIndex c) {
		fundamentalDomain.assign(c);
	}
	
	/** Become a periodic sequence of period <code>n</code> with all values
	 * equal to zero. */

	public void assignZero(int n) {
		fundamentalDomain.assignZero(n);
	}

	/** Keep the period but set all values to zero. */

	public void assignZero() {
		fundamentalDomain.assignZero();
	}

	public void setValueAt(int j, Complex x) {
		int n = fundamentalDomain.getNumIndices();
		int k = j%n;
		if (k<0) k+=n;
		fundamentalDomain.setValueAt(k, x);
	}
}

