/**
This file is part of a jTEM project.
All jTEM projects are licensed under the FreeBSD license 
or 2-clause BSD license (see http://www.opensource.org/licenses/bsd-license.php). 

Copyright (c) 2005-2010, Technische Universität Berlin, jTEM
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

-	Redistributions of source code must retain the above copyright notice, 
	this list of conditions and the following disclaimer.

-	Redistributions in binary form must reproduce the above copyright notice, 
	this list of conditions and the following disclaimer in the documentation 
	and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS 
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
OF SUCH DAMAGE.
**/

package de.jtem.function.real;


/** An implementation of the interface
* <code>RealFunction.PeriodicOnIndexCrossIntegers</code>. 
* Function values are stored internally, so using this class to make a copy of
* a <code>RealFunction.PeriodicOnIndexCrossIntegers <code>c</code> (by the corresponding
* constructor or the assign method) will make a snapshot of the current state
* of <code>c</code>. No reference to <code>c</code> itself is retained.*/

public class RealFunctionPeriodicOnIndexCrossIntegers
		implements RealFunction.PeriodicOnIndexCrossIntegers {

	RealFunctionOnIndexCrossIndex fundamentalDomain =
		new RealFunctionOnIndexCrossIndex();

	public int getNumRows() {
		return fundamentalDomain.getNumRows();
	}

	public int getPeriod() {
		return fundamentalDomain.getNumColumns();
	}

	public double valueAt(int i, int j) {
		int n = fundamentalDomain.getNumColumns();
		int s = j%n;
		if (s<0) s+=n;
		return fundamentalDomain.valueAt(i, s);
	}

	/** Creates a grid with 1 row and no columns. */

	public RealFunctionPeriodicOnIndexCrossIntegers() {
	}

	/** Creates a grid with <code>m</code> rows, period <code>n</code>
	 * and all values zero. */

	public RealFunctionPeriodicOnIndexCrossIntegers(int m, int n) {
		fundamentalDomain.assignZero(m, n);
	}

	/** Creates a copy of the function <code>c</code> */

	public RealFunctionPeriodicOnIndexCrossIntegers(
			RealFunction.PeriodicOnIndexCrossIntegers c) {
		assign(c);
	}

	/** Keep the number of rows and the period but set all values to zero. */

	public void assignZero() {
		fundamentalDomain.assignZero();
	}

	/** Become a matrix with <code>m</code> periodic rows of period
	 * <code>n</code> and all values zero.
	 * @param m must be at least one.
	 * @param n must be at least one. */

	public void assignZero(int m, int n) {
		fundamentalDomain.assignZero(m, n);
	}

	/** Do nothing if <code>m</code> is equal to the current number of rows and
	 * <code>n</code> is equal to the current period, become
	 * a matrix with <code>m</code> periodic rows of period code>n</code>
	 * and all values zero otherwise.
	 * @param m must be at least one.
	 * @param n must be at least one. */
	 
	public void setSize(int m, int n) {
		fundamentalDomain.setSize(m,n);
	}
	
	/** Become a copy of <code>c</code> */

	public void assign(final RealFunction.PeriodicOnIndexCrossIntegers c) {
		fundamentalDomain.assign(new RealFunction.OnIndexCrossIndex() {
			public int getNumRows() {
				return c.getNumRows();
			}
			public int getNumColumns() {
				return c.getPeriod();
			}
			public double valueAt(int i, int j) {
				return c.valueAt(i, j);
			}
		});
	}

	/** Become a copy of the periodic extension of <code>c</code> */

	public void assign(RealFunction.OnIndexCrossIndex c) {
		fundamentalDomain.assign(c);
	}

	/** Sets the value at index <code>i,j</code> to <code>z</code>.
	 * @param i must be non-negative and less than <code>getNumRows()</code>. */

	public void setValueAt(int i, int j, double z) {
		int n = fundamentalDomain.getNumColumns();
		int s = j%n;
		if (s<0) s+=n;
		fundamentalDomain.setValueAt(i, s, z);
	}
}

