/**
This file is part of a jTEM project.
All jTEM projects are licensed under the FreeBSD license 
or 2-clause BSD license (see http://www.opensource.org/licenses/bsd-license.php). 

Copyright (c) 2005-2010, Technische Universität Berlin, jTEM
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

-	Redistributions of source code must retain the above copyright notice, 
	this list of conditions and the following disclaimer.

-	Redistributions in binary form must reproduce the above copyright notice, 
	this list of conditions and the following disclaimer in the documentation 
	and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS 
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
OF SUCH DAMAGE.
**/

package de.jtem.function.real;


/** An implementation of the interface
* <code>RealFunction.TriplyPeriodicOnIntegersCrossIntegersCrossIntegers</code>. 
* Function values are stored
* internally, so using this class to make a copy of a
* <code>RealFunction.TriplyPeriodicOnIntegersCrossIntegersCrossIntegers c</code> (by the corresponding
* constructor or the assign method) will make a snapshot of the current state
* of <code>c</code>. No reference to <code>c</code> itself is retained. */

public class RealFunctionTriplyPeriodicOnIntegersCrossIntegersCrossIntegers
		implements RealFunction.TriplyPeriodicOnIntegersCrossIntegersCrossIntegers {

	RealFunctionOnIndexCrossIndexCrossIndex fundamentalDomain =
		new RealFunctionOnIndexCrossIndexCrossIndex();

	public int getThirdPeriod() {
		return fundamentalDomain.getNumLayers();
	}
	
	public int getSecondPeriod() {
		return fundamentalDomain.getNumRows();
	}

	public int getPeriod() {
		return fundamentalDomain.getNumColumns();
	}

	public double valueAt(int h, int i, int j) {
		int k = fundamentalDomain.getNumLayers();
		int q = h%k;
		if (q<0) q+=k;
		int m = fundamentalDomain.getNumRows();
		int r = i%m;
		if (r<0) r+=m;
		int n = fundamentalDomain.getNumColumns();
		int s = j%n;
		if (s<0) s+=n;
		return fundamentalDomain.valueAt(q, r, s);
	}

	/** Creates a grid with 1 layer, 1 row and 1 column and value zero. */

	public RealFunctionTriplyPeriodicOnIntegersCrossIntegersCrossIntegers() {
	}

	/** Creates a grid with <code>k</code> layers, <code>m</code> periodic rows,
	 * , <code>n</code> periodic columns and all values zero.
	 * @param k must be at least one.
	 * @param m must be at least one.
	 * @param n must be at least one. */

	public RealFunctionTriplyPeriodicOnIntegersCrossIntegersCrossIntegers(
			int k, int m, int n ) {
		fundamentalDomain.assignZero(k, m, n);
	}

	/** Creates a copy of the function <code>c</code> */

	public RealFunctionTriplyPeriodicOnIntegersCrossIntegersCrossIntegers(
			RealFunction.TriplyPeriodicOnIntegersCrossIntegersCrossIntegers c) {
		assign(c);
	}

	/** Creates a copy of the triply periodic extension of <code>c</code>. */

	public RealFunctionTriplyPeriodicOnIntegersCrossIntegersCrossIntegers(
			RealFunction.OnIndexCrossIndexCrossIndex c) {
		fundamentalDomain.assign(c);
	}

	/** Keep the number of layers and the periods but set all values to zero. */

	public void assignZero() {
		fundamentalDomain.assignZero();
	}

	/** Become a matrix with <code>m</code> layers, <code>m</code> periodic
	 * rows of period <code>n</code>, periodic columns of period
	 * <code>m</code> and all values zero.
	 * @param k must be at least one.
	 * @param m must be at least one.
	 * @param n must be at least one. */

	public void assignZero(int k, int m, int n) {
		fundamentalDomain.assignZero(k, m, n);
	}

	/** Do nothing if <code>k</code> is equal to the current third period,
	 * <code>m</code> is equal to the current second period and
	 * <code>n</code> is equal to the current period. Otherwise become
	 * a triply periodic matrix with <code>k</code> layers, <code>m</code> rows,
	 * <code>n</code> columns and all values zero.
	 * @param k must be at least one.
	 * @param m must be at least one.
	 * @param n must be at least one. */
	 
	public void setSize(int k, int m, int n) {
		fundamentalDomain.setSize(k,m,n);
	}
	
	/** Become a copy of <code>c</code> */

	public void assign(
		final RealFunction.TriplyPeriodicOnIntegersCrossIntegersCrossIntegers c) {
		fundamentalDomain.assign(new RealFunction.OnIndexCrossIndexCrossIndex() {
			public int getNumLayers() {
				return c.getThirdPeriod();
			}
			
			public int getNumRows() {
				return c.getSecondPeriod();
			}

			public int getNumColumns() {
				return c.getPeriod();
			}

			public double valueAt(int h, int i, int j) {
				return c.valueAt(h, i, j);
			}
		});
	}

	/** Become a copy of the triply periodic extension of <code>c</code> */

	public void assign(RealFunction.OnIndexCrossIndexCrossIndex c) {
		fundamentalDomain.assign(c);
	}

	/** Sets the value at index <code>h,i,j</code> to <code>z</code>. */

	public void setValueAt(int h, int i, int j, double z) {
		int k = fundamentalDomain.getNumLayers();
		int q = h%k;
		if (q<0) q+=k;
		int m = fundamentalDomain.getNumRows();
		int r = i%m;
		if (r<0) r+=m;
		int n = fundamentalDomain.getNumColumns();
		int s = j%n;
		if (s<0) s+=n;
		fundamentalDomain.setValueAt(q, r, s, z);
	}
}